import java.net.*;
import java.util.jar.*;
import java.util.Set;
import java.util.Iterator;
import java.security.interfaces.*;
import java.security.*;
import java.security.cert.Certificate;

public class Main {
    public static void main(String[] args) {
//
        try {
            URL url = new URL("jar:file:showfiles.jar!/ShowFiles.class");
            JarURLConnection jconn = (JarURLConnection)url.openConnection();

            System.out.println("JAR File URL: " + jconn.getJarFileURL());
                                                      // file:showfiles.jar

            System.out.println(jconn.getEntryName()); // ShowFiles.class

            // Print entry's attributes
            Attributes attrs = jconn.getAttributes();
            System.out.println("Attributes: " );
            if (attrs != null) {
                Set set = attrs.entrySet();
                for (Iterator it = set.iterator(); it.hasNext();) {
                    System.out.println(it.next());
                }
            }

            // This step is necessary; otherwise getCertificates() return null
            JarEntry entry = jconn.getJarEntry();
            JarFile jfile = jconn.getJarFile();
            java.io.InputStream is = jfile.getInputStream(entry);
            is.skip(entry.getSize());

            // Get and Print entry's certificates
            Certificate[] certs = jconn.getCertificates();
            System.out.println("Certificates: " );
            if (certs != null) {
                printCertificates(certs);
            }
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }
//
    }

    static void printCertificates(java.security.cert.Certificate[] certs) {
        for (int i=0; i<certs.length; i++) {
            System.out.println("  Type: "+certs[i].getType());
            if (certs[i] instanceof java.security.cert.X509Certificate) {
                java.security.cert.X509Certificate c =
                    (java.security.cert.X509Certificate)certs[i];
                    
                System.out.println("  Issuer: "+c.getIssuerDN().getName());
                System.out.println("  Subject: "+c.getSubjectDN().getName());
                System.out.println("  Serial#: "+c.getSerialNumber());
                System.out.println("  Expires: "+c.getNotAfter());
            }

            PublicKey pk = certs[i].getPublicKey();
            if (pk instanceof DSAPublicKey) {
                DSAPublicKey dpk = (DSAPublicKey)pk;
                System.out.println("  Algorithm: "+dpk.getAlgorithm());
                System.out.println("  Format: "+dpk.getFormat());
            }
        }
    }
}
