import java.util.*;

class Main {
    public static void main(String[] args) {
        Collection col = new FixedCol(3);

        System.out.println(col.add("dog"));     // true
        System.out.println(col.add("dog"));     // true
        System.out.println(col.add("cat"));     // true
        System.out.println(col);                // [dog, dog, cat]

        // The collection is full
        // System.out.println(col.add("pig"));  // IllegalStateException

        System.out.println(col.remove("pig"));  // false
        System.out.println(col.remove("dog"));  // true
        System.out.println(col.remove("dog"));  // true
        System.out.println(col);                // [cat]
    }
}

class FixedCol extends AbstractCollection {
    private Object[] array;
    int size = 0;

    FixedCol(int capacity) {
        array = new Object[capacity];
    }

    FixedCol() {
        this(10);  // Default capacity is 10
    }

    FixedCol(Collection c) {
        int cSize = c.size();
        array = new Object[(cSize*110)/100];  // Allow 10% for growth
        c.toArray(array);
        size = cSize;
    }

    public boolean add(Object e) {
        if (size == array.length)
            throw new IllegalStateException();
        array[size++] = e;
        return true;
    }

    public Iterator iterator() {
        // This is not a fail-fast iterator
        return new Iterator() {
            private int cursor = 0;
            private boolean removeProhibited = true;
            public boolean hasNext() {
                return cursor < size;
            }
            public Object next() {
                if (cursor==size)
                    throw new NoSuchElementException();
                removeProhibited = false;
                return array[cursor++];
            }
            public void remove() {
                if (removeProhibited)
                    throw new IllegalStateException();
                array[--cursor] = array[--size];
                array[size] = null;   // Eliminate "false reference"
                removeProhibited = true;
            }
        };
    }

    public int size() {
        return size;
    }
}
