import java.util.*;

class Main {
    public static void main(String[] args) {
        Set set = new FixedSet(2);

        System.out.println(set.add("dog"));     // true
        System.out.println(set.add("cat"));     // true
        System.out.println(set);                // [dog, cat]

        // The set is full
        // System.out.println(set.add("pig"));  // IllegalStateException

        // Add an identical element
        System.out.println(set.add("dog"));     // false
        System.out.println(set);                // [dog, cat]

        // Remove some elements.
        System.out.println(set.remove("pig"));  // false
        System.out.println(set.remove("cat"));  // true
        System.out.println(set);                // [dog]
    }
}

class FixedSet extends AbstractSet {
    private Object[] array;
    int size = 0;

    FixedSet(int capacity) {
        array = new Object[capacity];
    }

    FixedSet() {
        this(10);  // Default capacity is 10
    }

    FixedSet(Collection c) {
        array = new Object[(c.size()*110)/100];  // Allow >= 10% for growth
        addAll(c);
    }

    public boolean add(Object e) {
        if (contains(e))
            return false;
        if (size == array.length)
            throw new IllegalStateException();
        array[size++] = e;
        return true;
    }

    public Iterator iterator() {
        // This is not a fail-fast iterator
        return new Iterator() {
            private int cursor = 0;
            private boolean removeProhibited = true;
            public boolean hasNext() {
                return cursor < size;
            }
            public Object next() {
                if (cursor==size)
                    throw new NoSuchElementException();
                removeProhibited = false;
                return array[cursor++];
            }
            public void remove() {
                if (removeProhibited)
                    throw new IllegalStateException();
                array[--cursor] = array[--size];
                array[size] = null;   // Eliminate "false reference"
                removeProhibited = true;
            }
        };
    }

    public int size() {
        return size;
    }
}
