import java.io.*;
import java.util.*;
import java.util.jar.*;

class Main {
    public static void main(String[] args) {
        if (args.length < 1) {
            System.err.println("Usage: java Main <jar-file> [entry-name]");
            System.exit(1);
        } 

        try {
            JarFile jf = new JarFile(args[0], true);
            String entryName = 
                (args.length >= 2) ? args[1] : JarFile.MANIFEST_NAME;
            JarEntry je = jf.getJarEntry(entryName);

            if (je == null) {
                System.err.println("No such entry " + entryName);

                // List the names of all entries in the jar file.
                for (Enumeration e=jf.entries(); e.hasMoreElements(); ) {
                    System.out.println("  " + e.nextElement());
                }
                System.exit(1);
            }
                
            // Dump the jar entry data to standard output.
            InputStream is = jf.getInputStream(je);
            byte[] buf = new byte[1024];
            int len;
            while ((len = is.read(buf)) >= 0) {
                System.out.write(buf, 0, len);
            }
            is.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
